use serde::{Deserialize, Serialize};
use actix_web::{web, HttpRequest, HttpResponse, Responder};
use crate::models::models::IssueWithSummaryAndVotes;
use crate::repositories;
use crate::repositories::issue::{get_issue_vote_equity, get_vote_for_user, record_vote_for_issue, FilterOptions};
use crate::repositories::session::get_user_id;

#[derive(Serialize, Deserialize, Clone)]
enum IssueVote {
    Positive,
    Negative,
}

#[derive(Deserialize)]
pub struct CreateIssueRequest {
    title: String,
    paragraphs: Vec<String>,
}

#[derive(Serialize)]
pub struct CreateIssueResponse {
    title: String,
    paragraphs: Vec<String>,
}

pub async fn add_issue(req: HttpRequest, data: web::Json<CreateIssueRequest>) -> impl Responder {
    if let Some(session_id) = req.cookie("session") {
        if let Some(session) = repositories::session::get_session(session_id.value().to_string()) {
            repositories::issue::create_issue(&data.title, &data.paragraphs, &session.username);
            let resp = CreateIssueResponse {
                title: String::from(&data.title),
                paragraphs: data.paragraphs.clone(),
            };
            return HttpResponse::Ok().json(resp);
        }
        HttpResponse::Unauthorized().into()
    } else {
        HttpResponse::Unauthorized().into()
    }
}

#[derive(Deserialize)]
pub struct IssuesPaginationRequest {
    min_positive_votes: Option<i64>,
    min_votes: Option<i64>,
    offset: i32,
    limit: i16,
}

#[derive(Serialize)]
pub struct IssuesPaginationResponse {
    issues: Vec<IssueWithSummaryAndVotes>,
}

pub async fn list_issues(req: web::Query<IssuesPaginationRequest>) -> impl Responder {
    let issues = repositories::issue::get_issues(
        Option::from(FilterOptions {
            min_positive_votes: req.min_positive_votes,
            min_votes: req.min_votes,
        }),
        req.offset,
        req.limit
    );
    HttpResponse::Ok().json(IssuesPaginationResponse { issues })
}

#[derive(Deserialize)]
pub struct GetParagraphsRequest {
    issue_id: i32,
}

#[derive(Serialize)]
pub struct GetParagraphsResponse {
    paragraphs: Vec<Option<String>>,
}

pub async fn get_paragraphs(query: web::Query<GetParagraphsRequest>) -> impl Responder {
    let paragraphs = repositories::issue::get_paragraphs(query.issue_id);
    HttpResponse::Ok().json(GetParagraphsResponse { paragraphs })
}

#[derive(Deserialize, Clone)]
pub struct VoteIssueRequest {
    issue_id: i32,
    vote: IssueVote,
}

#[derive(Serialize)]
pub struct VoteIssueResponse {
    issue_id: i32,
    equity: i64,
    positive: Option<bool>,
}

pub async fn vote_issue(req: HttpRequest, data: web::Json<VoteIssueRequest>) -> impl Responder {
    let positive_vote = match data.vote {
        IssueVote::Positive => true,
        IssueVote::Negative => false,
    };
    if let Some(session_id) = req.cookie("session") {
        if let Some(user_id) = get_user_id(session_id.value().to_string()) {
            record_vote_for_issue(positive_vote, data.issue_id, user_id);
            let resp = VoteIssueResponse {
                issue_id: data.issue_id,
                equity: get_issue_vote_equity(data.issue_id),
                positive: match data.vote {
                    IssueVote::Positive => Some(true),
                    IssueVote::Negative => Some(false),
                },
            };
            HttpResponse::Ok().json(resp)
        } else {
            HttpResponse::Unauthorized().into()
        }
    } else {
        HttpResponse::Unauthorized().into()
    }
}

#[derive(Deserialize)]
pub struct GetUserVoteRequest {
    issue_id: i32,
}

#[derive(Serialize)]
pub struct GetUserVoteResponse {
    vote: Option<IssueVote>,
}

pub async fn get_user_vote(req: HttpRequest, query: web::Query<GetUserVoteRequest>) -> impl Responder {
    let mut resp = GetUserVoteResponse { vote: None };
    if let Some(session_cookie) = req.cookie("session") {
        return if let Some(user_id) = get_user_id(session_cookie.value().to_string()) {
            let vote = get_vote_for_user(query.issue_id, user_id);
            resp = GetUserVoteResponse {
                vote: match vote {
                    Some(val) => {
                        if val {
                            Some(IssueVote::Positive)
                        } else {
                            Some(IssueVote::Negative)
                        }
                    },
                    None => None,
                }
            };
            HttpResponse::Ok().json(resp)
        } else {
            HttpResponse::Ok().json(GetUserVoteResponse { vote: None })
        }
    }
    HttpResponse::Ok().json(resp)
}